#ifndef _IePadvision_
#define _IePadvision_

typedef unsigned char BYTE;
#if defined(LINUX)
#pragma pack(push)
#pragma pack(1)
#endif

class IePadvision 
{
public:
	
	typedef void (*callbackfunction)(long abs_x, long abs_y, long pressure, unsigned char fTouch, unsigned char fTap, long buttonState, unsigned long dwTimeStamp, unsigned long dwSequenceNum );
	typedef void (*widgeteventcallbackfunction)(unsigned char widgetid, unsigned char widgetType, unsigned char widgetCode );
	/* define the LCD range in pixel.
	 * the range is defined by its up-left point and its width and height */
	typedef struct StructLCDPixelRange
	{
		/* ths x-axis pixel index of the up-left point */
		unsigned short X;
		/* ths y-axis pixel index of the up-left point */
		unsigned short Y;
		/* the range width in pixel */
		unsigned short Width;
		/* the range height in pixel */
		unsigned short Height;	
	} StructLCDPixelRange;

	/* define the LCD X-axis and Y-axis pixel numbers */
	typedef struct StructLCDPixels
	{
		/* ths x-axis pixel index of the up-left point */
		unsigned short X;
		/* ths y-axis pixel index of the up-left point */
		unsigned short Y;
	} StructLCDPixels;
	
	typedef struct StructLCDColor
	{
		unsigned char Alpha;	
		unsigned char Blue;
		unsigned char Green;
		unsigned char Red;
	} StructLCDColor;
	
	//enum class ReportRateIndicator
	typedef enum ReportRateIndicator
	{
		RPT400PS = 1,
		RPT200PS = 2,
		RPT100PS = 4
	} ReportRateIndicator;

	typedef struct StructInkRegion 
	{
		bool Enabled;
		bool ShowInkTrails;
		bool Transform;
		unsigned short Height;
		unsigned short Width;
		unsigned short Top;
		unsigned short Left;
		StructLCDColor PenColor;
		short PenWidth;
		short InkTrailsTimeout;
	} StructInkRegion;
	
	//enum class WidgetType:unsigned char  {
	typedef enum WidgetType {
		WT_TEXT = 3,
		WT_BUTTON = 4,
		WT_CHECKBOX = 5,
		WT_RADIOBUTTON = 6
	} WidgetType;
	
	typedef enum EventType  {
		TEXT_Event = 3,
		BUTTON_Event = 4,
		CHECKBOX_Event = 5,
		RADIOBUTTON_Event = 6
	} EventType;

	//enum class WidgetEventCode: unsigned char
	typedef enum WidgetEventCode
	{
		//the widget has been created
		CREATE_SUCCEED = 1,
		//The widget has been deleted
		DELETE_WIDGET_DONE,
		//the widget has been checked (for checkbox, radio button)
		WIDGET_CHECKED,
		//the widget has been unchecked (for checkbox, radio button)
		WIDGET_UNCHECKED,
		//the widget has been clicked
		WIDGET_CLICKED,
		//the widget has been moved
		WIDGET_MOVE_DONE,
		//the widget has been resized
		WIDGET_RESIZE_DONE,
		//the widget has been enabled
		WIDGET_ENABLED,
		//the widget has been disabled
		WIDGET_DISABLED,
		//the widget has been shown
		WIDGET_SHOW,
		//the widget has been hidden
		WIDGET_HIDE,
		//the widget text has been changed
		WIDGET_TEXT_DONE,
		//the widget background color has been changed
		WIDGET_BKCOLOR_CHANGED,
		//the widget foreground color has been changed
		WIDGET_FGCOLOR_CHANGED,
		//the widget appearance effect has been changed
		WIDGET_EFFECT_CHANGED,
		//the widget font has been changed
		WIDGET_FONT_CHANGED,
		//skip calibrarion events
		//belows are error codes
		//invalid widget id
		WIDGET_INVALID_ID=150,
		//invalid type
		WIDGET_INVALID_TYPE,
		//invald request
		WIDGET_INVALID_REQUEST,
		//invalid font id
		WIDGET_INVALID_FONT,
		//Create_failed
		WIDGET_CREATE_FAILED
	} WidgetEventCode;

	//enum class WIDGET_FUNCTION:unsigned char 
	typedef enum WIDGET_FUNCTION 
	{
		 /*******************************************************************************
		 * Function operator (2 bits) defined as  below
		 *		00:  Create a widget
		 *  	40:  Delete an existed if applicable and then create a new one
		 *  	80:  Delete the widget
		 *  	c0:  Change widget attributes
		 *******************************************************************************/
		WF_CREATE    = 0x00,
		WF_RECREATE  = 0x40,
		WF_DELETE    = 0x80,
		WF_CHANGE    = 0xC0,

		/// <summary>
		/// The bit mask
		/// </summary>
		WF_OPERATIONFILTER = 0xC0
	} WIDGET_FUNCTION;
	
	//Table for font name and its id
	typedef struct struct_FontNameId {
		 const char * fontname;
		 const BYTE fontid;
	} FontNameID;

	//Table for PinPad FieldName and its id
	typedef struct struct_PinPadFieldId {
		 const char * field;
		 const BYTE ppid;
	} PinPadFieldID;

	/*
	 * defined Widget Effect
	 */
	//enum class WidgetEffect: unsigned char {
	typedef enum WidgetEffect {
		/* 3D appearance */
		WE_THREED = 0,
		WE_SIMPLE,
		WE_NONE
	}WidgetEffect;

	/* defined the Widget Info structure
	 */
	 typedef struct StructWidgetInfoData
	 {
		 //The Widget ID, from 1 - 200
		 BYTE WidgetID;
		 //The widget type
		 IePadvision::WidgetType wdType;
		 //The font ID
		 BYTE FontID;
		 //Color of the Widget background
		 StructLCDColor BgColor;
		 //Color of the Widget foreground
		 StructLCDColor FgColor;
		 //The Widget's height, from 20 - 272 pixels
		 unsigned short Height;
		 //The Widget's Width, from 40 - 480 pixels
		 unsigned short Width;
		 //The y corrdinate value of widget's top-left position
		 unsigned short Top;
		 //The x corrdinate value of widget's top-left position
		 unsigned short Left;
		 //widget appearance effect
		 WidgetEffect Effect;
		 //group id, valid value from 1 to 255, if 0 is given, the incurred radio button will not be grouped to any other.
		 //only works for RADIOButton. other type, it will be ignored.
		 BYTE GroupID;
		 //widget text
		 char Text[960];
	 } StructWidgetInfo;
 
	//defines system font id
	//enum class FontID: unsigned char {
	typedef enum FontID {
		 SAN_R_13 = 0, /* San Serif 13 */
		 SYS_FIRSTFONT_ID = 0,
		 SAN_B_13 = 1, /* Bold San Serif 13 */
		 SAN_R_16 = 2, /* San Serif 16 */
		 SAN_B_16 = 3, /* Bold San Serif 16 */
		 SAN_R_20 = 4, /* San Serif 20 */
		 SAN_B_20 = 5, /* Bold San Serif 20 */
		 SAN_R_24 = 6, /* San Serif 24 */
		 SAN_B_24 = 7, /* Bold San Serif 24 */
		 SAN_R_32 = 8, /* San Serif 32 */
		 SAN_B_32 = 9, /* Bold San Serif 32 */
		 MON_B_08 = 10, /* Bold Monospaced 8 */
		 MON_B_09 = 11, /* Bold Monospaced 9 */
		 MON_B_10 = 12, /* Bold Monospaced 10 */
		 MON_B_12 = 13, /* Bold Monospaced 12 */
		 MON_R_13 = 14, /* Monospaced 13 */
		 MON_B_15 = 15, /* Bold Monospaced 15 */
		 MON_B_16 = 16, /* Bold Monospaced 16 */
		 MON_B_17 = 17, /* Bold Monospaced 17 */
		 MON_B_18 = 18, /* Bold Monospaced 18 */
		 MON_B_48 = 19, /* Bold Monospaced 48 */
		 COM_B_18 = 20, /* Bold Comic 18 */
		 COM_B_24 = 21  /* Bold Comic 24 */
	 }FontID;
 
	 //defines pin pad widget control id
	 //enum class PinPadControls: unsigned char {
	typedef enum PinPadControls {
		 PPC_NUM_10 = 0, //The numeric key for number 0
		 PPC_NUM_1 = 1,
		 PPC_NUM_2 = 2,
		 PPC_NUM_3 = 3,
		 PPC_NUM_4 = 4,
		 PPC_UM_5 = 5,
		 PPC_NUM_6 = 6,
		 PPC_NUM_7 = 7,
		 PPC_NUM_8 = 8,
		 PPC_NUM_9 = 9, //The numeric key for number 9
		 PPC_FUN_BACKSPACE = 11, //The functional key to backspace
		 PPC_FUN_SEC = 30, //the functional checkbox to show the numbers as asterisks
		 PPC_FUN_OK = 40, //the functional button to accept and dismiss the PinPad
		 PPC_FUN_CANCEL = 50, //the functional button to cancel and dismiss the PinPad
		 PPC_CAPTION = 19, //text field to show customized caption
		 PPC_CONTENT = 20, //text field to display the entered numeric string
		 PPC_IMAGE = 60 //CUSTOM IMAGE
	 }PinPadControls;
	 
	 struct struct_PPCNameId {
		 char *ppcname;
		 BYTE ppcid;
	 };
 
private:

	callbackfunction m_dataevent;
	widgeteventcallbackfunction m_widgetevent;
	

public:
		IePadvision();
		~IePadvision();
		virtual void CloseConnection();
		virtual bool OpenConnection();
		virtual short GetNumDevices();
		virtual char *GetSerialNumber();
		virtual bool RegisterEventHandlers(callbackfunction ptrfn, widgeteventcallbackfunction ptrwfn);
		virtual double GetPhysicalX();
		virtual double GetPhysicalY();
		virtual bool GetDataRanges(int *minx, int *maxx, int *miny, int *maxy, int *minp, int *maxp);

		//extra methods
		//Get LCDPixel x,y number
		virtual int GetLCDPixelsStruct(IePadvision::StructLCDPixels *lcdpixels);
		//Get inking region
		virtual int GetInkRegionStruct(IePadvision::StructInkRegion *pRegion);

		//Set inking region
		virtual bool SetInkRegion(IePadvision::StructInkRegion Region);
		//Clear inking region area
		virtual bool ClearInkRegion();
		//Get ink report rate
		virtual int GetReportRate(unsigned char *value);
		//Set ink report rate
		virtual bool SetReportRate(unsigned char value);
		//Set Screen Background Color
		virtual bool SetBackGroundColor(IePadvision::StructLCDColor lcolor);
		//Reset to system default setting
		virtual bool ResetSystemDefaults();
		//Set specified image as the background image by Onoff value set to 1
		virtual bool EnableBackgroundImage(unsigned char Onoff);
		//Get specified image whether a background image, return the current state in Onoff
		virtual bool BackgroundImageState(unsigned char *Onoff);
		//Download the background image from imagepath at the location defined by range
		virtual int SetBackgroundImage(IePadvision::StructLCDPixelRange range, char *imagepath);
		//Set specified image as the custom background image by Onoff value set to 1
		virtual bool EnableCustomBackgroundImage(unsigned char Onoff);
		//Get specified image wheter custom background image, return the current state in Onoff
		virtual bool CustomBackgroundImageState(unsigned char *Onoff);
		//Download the custom background image from imagepath at the location defined by range
		virtual int SetCustomBackgroundImage(char *imageid, IePadvision::StructLCDPixelRange range, char *image_path);
		//Retrieve the custom background image id from device
		virtual int CustomBackgroundImageID(char *imageid);
		//Widget Creation
		virtual bool WidgetCreate(StructWidgetInfo winfo);
		//Widget Enable/Disable
		virtual bool EnableWidget(BYTE widgetID, bool bEnable);
		//Delete a Widget
		virtual bool DeleteWidget(BYTE widgetID);
		//Widget Show/Hide
		virtual bool ShowWidget(BYTE widgetID, bool bShow);
		//Remove all the Widgets, and setup PinPad Mode
		virtual bool RemoveAllWidget(bool bSetPinPadFlag);
		//Move Widget to new location
		virtual bool MoveWidget(BYTE widgetID, short xPos, short yPos);
		//Check/Uncheck Widget
		virtual bool SetWidgetChecked(BYTE widgetID, bool bCheck);
		//modified widget text
		virtual bool SetWidgetText(BYTE widgetID, char *text);
		//Load Widget Layout from xmlfile
		virtual bool LoadWidgetLayout(char *xmlfile);
		//Dispaly PinPad and put the return value in retstring
		virtual int DisplayPinPad(char *xmlfile, int timeout_value, char *retstring);
		//Screen saver 
		//Screen saver time out value to luanch screensaver, range from 1 to 60 seconds
		int m_SS_timeout;
		//the interval value for the image to display
		int m_SS_interval;
		//Whether the screen saver is enabled
		bool m_SS_enabled;
		//Screen Saver Images count in the device
		short m_SS_imagecount;
		//whether screen saver run on stand alone situation
		bool m_SS_standalone;
		//whether the screensaver is running now
		bool SS_isRunning();
		//Add an additional image from imagepath to the driver with specified its index which range from 0-9
		virtual int SS_addImage(int index, char * imagepath);
		//Delete Image from the driver
		virtual bool SS_deleteImage(int index);
		//Stop the screensaver if it is running
		virtual bool SS_stop();
		//Resume the screensaver
		virtual bool SS_resume();
		//Reset all image and restore to default settings
		virtual void SS_reset();
		//Show the indexed image
		virtual bool SS_imagePreview(int index);
		//Store Screen Saver setting to device
		virtual bool SS_storeSettings();
		//Retrieve screen saver settings from device
		virtual bool SS_retrieveSettings();
		//set screen saver related variables to default number
		virtual void SS_defaults(bool init = false);
};


#endif /* _IePadvision */