#ifndef _EPADVISION_USB_
#define _EPADVISION_USB_

#define EPADVISION_CLASSID "3F2DE94B14D448719DFE5359EF03652A"
#define EPADVISION_DES "ePad Vision"
#define EPADVISION_ICA_NAME "epadvis"
#define EPADLINKVID 0x04df

#include "../ieusbio/ieusb_io.h"

#include <signal.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <semaphore.h>
#include <pthread.h>
#include <errno.h>

#include "../ieusbio/io_linux.h"
#include "/usr/include/linux/usbdevice_fs.h"
#include "../ieusbio/usb_io.h"
#include "../ieusbio/usb_vision_io.h"
#include "common.h"

#include "epadvisionutil.h"
#include <libxml/xmlreader.h>

#include "CImg-1.5.7/CImg.h"
using namespace cimg_library;
#if 0 //leave here for future reference
#include "opencv2/core/core.hpp"
#include "opencv2/highgui/highgui.hpp"
#endif

typedef struct tagVISREPORT_DATA {
    DWORD length;
    BYTE  data[20];
} VISREPORT_DATA, *PVISREPORT_DATA;

typedef struct tagVISDATA_RECORD {
    VISREPORT_DATA report;
    struct tagVISDATA_RECORD *pNext;
} VISDATA_RECORD, *PVISDATA_RECORD;


/*
/// <summary>
/// Enumerate the widget types
/// </summary>
*/
enum class WidgetType:unsigned char  {
	TEXT = 3,
	BUTTON = 4,
	CHECKBOX = 5,
	RADIOBUTTON = 6
};

enum class EventType:unsigned char  {
	TEXT_Event = 3,
	BUTTON_Event = 4,
	CHECKBOX_Event = 5,
	RADIOBUTTON_Event = 6
};

enum class WidgetEventCode: unsigned char
{
	//the widget has been created
	CREATE_SUCCEED = 1,
	//The widget has been deleted
	DELETE_WIDGET_DONE,
	//the widget has been checked (for checkbox, radio button)
	WIDGET_CHECKED,
	//the widget has been unchecked (for checkbox, radio button)
	WIDGET_UNCHECKED,
	//the widget has been clicked
	WIDGET_CLICKED,
	//the widget has been moved
	WIDGET_MOVE_DONE,
	//the widget has been resized
	WIDGET_RESIZE_DONE,
	//the widget has been enabled
	WIDGET_ENABLED,
	//the widget has been disabled
	WIDGET_DISABLED,
	//the widget has been shown
	WIDGET_SHOW,
	//the widget has been hidden
	WIDGET_HIDE,
	//the widget text has been changed
	WIDGET_TEXT_DONE,
	//the widget background color has been changed
	WIDGET_BKCOLOR_CHANGED,
	//the widget foreground color has been changed
	WIDGET_FGCOLOR_CHANGED,
	//the widget appearance effect has been changed
	WIDGET_EFFECT_CHANGED,
	//the widget font has been changed
	WIDGET_FONT_CHANGED,
	//skip calibrarion events
	//belows are error codes
	//invalid widget id
	WIDGET_INVALID_ID=150,
	//invalid type
	WIDGET_INVALID_TYPE,
	//invald request
	WIDGET_INVALID_REQUEST,
	//invalid font id
	WIDGET_INVALID_FONT,
	//Create_failed
	WIDGET_CREATE_FAILED
} ;

enum class WIDGET_FUNCTION:unsigned char 
{
	 /*******************************************************************************
	 * Function operator (2 bits) defined as  below
	 *		00:  Create a widget
	 *  	40:  Delete an existed if applicable and then create a new one
	 *  	80:  Delete the widget
	 *  	c0:  Change widget attributes
	 *******************************************************************************/
	F_CREATE    = 0x00,
	F_RECREATE  = 0x40,
	F_DELETE    = 0x80,
	F_CHANGE    = 0xC0,

	/// <summary>
	/// The bit mask
	/// </summary>
	F_OPERATIONFILTER = 0xC0
} ;

enum class ReportRateIndicator
{
	RPT400PS = 1,
	RPT200PS = 2,
	RPT100PS = 4
};


struct InkRegion 
{
	bool Enabled;
	bool ShowInkTrails;
	bool Transform;
	unsigned short Height;
	unsigned short Width;
	unsigned short Top;
	unsigned short Left;
	LCDColor PenColor;
	short PenWidth;
	short InkTrailsTimeout;
} ;

/* define the LCD range in pixel.
 * the range is defined by its up-left point and its width and height */
struct LCDPixelRange
{
	/* ths x-axis pixel index of the up-left point */
	INT16U X;
	/* ths y-axis pixel index of the up-left point */
	INT16U Y;
	/* the range width in pixel */
	INT16U Width;
	/* the range height in pixel */
	INT16U Height;	
};

/* define the LCD X-axis and Y-axis pixel numbers */
struct LCDPixels
{
	/* ths x-axis pixel index of the up-left point */
	INT16U X;
	/* ths y-axis pixel index of the up-left point */
	INT16U Y;
};

//Table for font name and its id
typedef struct struct_FontNameId {
	 const char * fontname;
	 const BYTE fontid;
} FontNameID;

//Table for PinPad FieldName and its id
typedef struct struct_PinPadFieldId {
	 const char * field;
	 const BYTE ppid;
} PinPadFieldID;

/*
 * defined Widget Effect
 */
enum class WidgetEffect: unsigned char {
	/* 3D appearance */
	THREED = 0,
	SIMPLE,
	NONE
};

/* defined the Widget Info structure
 */
 typedef struct WidgetInfoData
 {
	 //The Widget ID, from 1 - 200
	 BYTE WidgetID;
	 //The widget type
	 WidgetType wdType;
	 //The font ID
	 BYTE FontID;
	 //Color of the Widget background
	 LCDColor BgColor;
	 //Color of the Widget foreground
	 LCDColor FgColor;
	 //The Widget's height, from 20 - 272 pixels
	 unsigned short Height;
	 //The Widget's Width, from 40 - 480 pixels
	 unsigned short Width;
	 //The y corrdinate value of widget's top-left position
	 unsigned short Top;
	 //The x corrdinate value of widget's top-left position
	 unsigned short Left;
	 //widget appearance effect
	 WidgetEffect Effect;
	 //group id, valid value from 1 to 255, if 0 is given, the incurred radio button will not be grouped to any other.
	 //only works for RADIOButton. other type, it will be ignored.
	 BYTE GroupID;
	 //widget text
	 char Text[960];
 } WidgetInfo;
 
 //defines system font id
 enum class FontID: unsigned char {
	 SAN_R_13 = 0, /* San Serif 13 */
	 SYS_FIRSTFONT_ID = 0,
	 SAN_B_13 = 1, /* Bold San Serif 13 */
	 SAN_R_16 = 2, /* San Serif 16 */
	 SAN_B_16 = 3, /* Bold San Serif 16 */
	 SAN_R_20 = 4, /* San Serif 20 */
	 SAN_B_20 = 5, /* Bold San Serif 20 */
	 SAN_R_24 = 6, /* San Serif 24 */
	 SAN_B_24 = 7, /* Bold San Serif 24 */
	 SAN_R_32 = 8, /* San Serif 32 */
	 SAN_B_32 = 9, /* Bold San Serif 32 */
	 MON_B_08 = 10, /* Bold Monospaced 8 */
	 MON_B_09 = 11, /* Bold Monospaced 9 */
	 MON_B_10 = 12, /* Bold Monospaced 10 */
	 MON_B_12 = 13, /* Bold Monospaced 12 */
	 MON_R_13 = 14, /* Monospaced 13 */
	 MON_B_15 = 15, /* Bold Monospaced 15 */
	 MON_B_16 = 16, /* Bold Monospaced 16 */
	 MON_B_17 = 17, /* Bold Monospaced 17 */
	 MON_B_18 = 18, /* Bold Monospaced 18 */
	 MON_B_48 = 19, /* Bold Monospaced 48 */
	 COM_B_18 = 20, /* Bold Comic 18 */
	 COM_B_24 = 21  /* Bold Comic 24 */
 };
 
 
 
 
 //defines pin pad widget control id
 enum class PinPadControls: unsigned char {
	 NUM_10 = 0, //The numeric key for number 0
	 NUM_1 = 1,
	 NUM_2 = 2,
	 NUM_3 = 3,
	 NUM_4 = 4,
	 NUM_5 = 5,
	 NUM_6 = 6,
	 NUM_7 = 7,
	 NUM_8 = 8,
	 NUM_9 = 9, //The numeric key for number 9
	 FUN_BACKSPACE = 11, //The functional key to backspace
	 FUN_SEC = 30, //the functional checkbox to show the numbers as asterisks
	 FUN_OK = 40, //the functional button to accept and dismiss the PinPad
	 FUN_CANCEL = 50, //the functional button to cancel and dismiss the PinPad
	 CAPTION = 19, //text field to show customized caption
	 CONTENT = 20, //text field to display the entered numeric string
	 IMAGE = 60 //CUSTOM IMAGE
 };
 
 struct struct_PPCNameId {
	 char *ppcname;
	 BYTE ppcid;
 };
 
typedef void (*callbackfunction)(long abs_x, long abs_y, long pressure, unsigned char fTouch, unsigned char fTap, long buttonState, unsigned long dwTimeStamp, unsigned long dwSequenceNum );
typedef void (*widgeteventcallbackfunction)(BYTE widgetid, EventType widgetType, WidgetEventCode widgetCode );
 
class epadvision
{
 
	private:
	protected:
		IEUSB_LIB_HANDLE m_libhandle;
		IEUSB_HANDLE m_handle;
		sigset_t signal_set;
		INPUT_BUFFER inputBuffer[1]; 
		struct usbdevfs_urb inputURB[sizeof(inputBuffer)/sizeof(inputBuffer[0])];
		ULONG m_visDataSize;
		int gRunning;
		static void * procdata_thread(void * arg);
		void procdata_routine();
		void usrSignalHandler(int iValue);
		void polldata_routine();
		static void * polldata_thread(void * arg);
		int GetPadInfo(DevicePadInfo *padinfo);
		int SetInkTraceOpt(int OnOff, int timeout);
		bool ResetePadVisionDevice();	
		int SetMode(BYTE mode);
		int GetMode(BYTE* mode);
        //creating two threads to poll the data from device and put the 
        //data into data queue. The "processdata" thread retrieves the data 
        //from queue and process it.
        int status;
        pthread_t polldata;
        pthread_t processdata;
		
		struct thread_struct {
			epadvision *This;
			void *thread_arg;
			thread_struct (epadvision * t, void *p)
				:This(t), thread_arg(p){}
		};
		short OpenConnection();
		DevicePadInfo m_padinfo;
		InkRegion m_inkregion;
		LCDPixels m_lcdpixels;
		bool m_bFirstPkt;
		struct thread_struct *m_proc_arg;
		int WriteImageData(LCDPixelRange range, char *image_path, int imgindex, bool isBKImage, char *guid);
		epadvisionutil *m_util;
		//Handling xml file
		int processNode(xmlTextReaderPtr reader, bool bIsPinPad);
		void streamFile(const char *filename, bool bIsPinPad);
		void xmlcleanup();
		FontNameID m_fontTable[22];
		PinPadFieldID m_ppfieldTable[17];
		pthread_mutex_t m_pinpadmutex;
		bool m_PinPad;
		bool setPinPad(bool value);
		bool getPinPad();
		bool LoadWidgetLayout(char *xmlfile, bool bIsPinPad);
		int m_currentWID;
		char m_pinpadstr[128];
		int m_nIndex;
		pthread_mutex_t m_Widget_mutex;
		pthread_cond_t m_Widget_condv;
		bool setEvent();
		bool waitEvent(int seconds);
		void setcurrentWID(BYTE value);
		BYTE getcurrentWID();
		bool m_SS_bStop;
		char * m_SS_Images[10]; 
		
	public:
		epadvision();
		virtual ~epadvision();
		/**
		User-defined data callback handler.
		Context is set by user when constructing a CEPadIoEngine object.
		The buffer passed in contains the data -- note that this is the *same* buffer that is used by the object and
		will be re-used as soon as the user-defined routine returns.
		*/
		typedef void (*DATA_HANDLER)(void *pContext, __u8 *pBuffer);
		//gIIePad interface methods:
		short GetNumDevices();
		short OpenConnection(callbackfunction ptrfn);
		callbackfunction m_callbackfn;
		widgeteventcallbackfunction m_widgetevent;
		void CloseConnection();
		double GetPhysicalX();
		double GetPhysicalY();
		bool GetSerialNumber(char *serial);
		bool GetDataRanges(int *minx, int *maxx, int *miny, int *maxy, int *minp, int *maxp);
		//extra methods
		//Get LCDPixel x,y number
		int GetLCDPixels(LCDPixels *lcdpixels);
		//Get inking region
		int GetInkRegion(InkRegion *pRegion);
		//Set inking region
		bool SetInkRegion(InkRegion Region);
		//Clear inking region area
		bool ClearInkRegion();
		//Get ink report rate
		int GetReportRate(BYTE *value);
		//Set ink report rate
		bool SetReportRate(BYTE value);
		//Set Screen Background Color
		bool SetBackGroundColor(LCDColor lcolor);
		//Reset to system default setting
		bool ResetSystemDefaults();
		//Set specified image as the background image by Onoff value set to 1
		bool EnableBackgroundImage(BYTE Onoff);
		//Get specified image whether a background image, return the current state in Onoff
		bool BackgroundImageState(BYTE *Onoff);
		//Download the background image from imagepath at the location defined by range
		int SetBackgroundImage(LCDPixelRange range, char *imagepath);
		//Set specified image as the custom background image by Onoff value set to 1
		bool EnableCustomBackgroundImage(BYTE Onoff);
		//Get specified image wheter custom background image, return the current state in Onoff
		bool CustomBackgroundImageState(BYTE *Onoff);
		//Download the custom background image from imagepath at the location defined by range
		int SetCustomBackgroundImage(char *imageid, LCDPixelRange range, char *image_path);
		//Retrieve the custom background image id from device
		int CustomBackgroundImageID(char *imageid);
		//Widget features need to open with widgetevent callback function
		short OpenConnection(callbackfunction ptrfn, widgeteventcallbackfunction widgetptrfn);
		//Widget Creation
		bool WidgetCreate(WidgetInfo winfo);
		//Widget Enable/Disable
		bool EnableWidget(BYTE widgetID, bool bEnable);
		//Delete a Widget
		bool DeleteWidget(BYTE widgetID);
		//Widget Show/Hide
		bool ShowWidget(BYTE widgetID, bool bShow);
		//Remove all the Widgets, and setup PinPad Mode
		bool RemoveAllWidget(bool bSetPinPadFlag);
		//Move Widget to new location
		bool MoveWidget(BYTE widgetID, short xPos, short yPos);
		//Check/Uncheck Widget
		bool SetWidgetChecked(BYTE widgetID, bool bCheck);
		//modified widget text
		bool SetWidgetText(BYTE widgetID, char *text);
		//Load Widget Layout from xmlfile
		bool LoadWidgetLayout(char *xmlfile);
		//Dispaly PinPad and put the return value in retstring
		int DisplayPinPad(char *xmlfile, int timeout_value, char *retstring);
		//Screen saver 
		//Screen saver time out value to luanch screensaver, range from 1 to 60 seconds
		int m_SS_timeout;
		//the interval value for the image to display
		int m_SS_interval;
		//Whether the screen saver is enabled
		bool m_SS_enabled;
		//Screen Saver Images count in the device
		short m_SS_imagecount;
		//whether screen saver run on stand alone situation
		bool m_SS_standalone;
		//whether the screensaver is running now
		bool SS_isRunning();
		//Add an additional image from imagepath to the driver with specified its index which range from 0-9
		int SS_addImage(int index, char * imagepath);
		//Delete Image from the driver
		bool SS_deleteImage(int index);
		//Stop the screensaver if it is running
		bool SS_stop();
		//Resume the screensaver
		bool SS_resume();
		//Reset all image and restore to default settings
		void SS_reset();
		//Show the indexed image
		bool SS_imagePreview(int index);
		//Store Screen Saver setting to device
		bool SS_storeSettings();
		//Retrieve screen saver settings from device
		bool SS_retrieveSettings();
		//set screen saver related variables to default number
		void SS_defaults(bool init = false);
};
#endif /* _EPADVISION_ */